/*
 * Copyright (c) 1997-2010 EMC Corporation. All rights reserved.
 *
 * This file shall only be used to demonstrate how to interface to an
 * EMC Corporation licensed development product.
 *
 * You have a royalty-free right to use, reproduce and distribute this
 * demonstration file, provided that you agree that EMC Corporation has no
 * warranty, implied or otherwise, or liability for this demonstration file
 * (including any modified version).  This software is provided "as is"
 * without warranties or representations of any kind. EMC Corporation
 * disclaims all conditions and warranties, statutory and otherwise, both
 * express and implied, with respect to the software, its quality and
 * performance, including but not limited to, all implied warranties of
 * merchantability, fitness for a particular purpose, title and
 * noninfringement of third party rights. Without limiting the foregoing,
 * EMC Corporation does not warrant that the software is error-free or that
 * errors in the product will be corrected. You agree that EMC Corporation
 * shall not be liable for any direct, indirect, incidental, special,
 * consequential, punitive or other damages whatsoever resulting from your use
 * of this software or any modified version.
 *
 */
package jce.common;

import java.io.IOException;
import java.io.InputStream;
import java.security.NoSuchProviderException;
import java.security.cert.CRLException;
import java.security.cert.Certificate;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.security.cert.X509CRL;
import java.security.cert.X509Certificate;

import util.ResourceReader;

/*
 *
 * This file contains some methods that make working with Certificates a
 * bit easier. This code is intended to make the certificate path sample code
 * more readable by removing unnecessary code.
 * <p>
 * Feel free, however, to adopt the code contained within for your own purposes.
 */
public final class CertUtil {

    private CertUtil() {
        // Prevent instantiation.
    }

    /**
     * Load a Certificate object from a specified file.
     *
     * @param filename The name of the file containing the X.509 Certificate.
     * @return A <code>Certificate</code> object containing the certificate.
     * @throws Exception On failure.
     */
    public static Certificate loadCertificate(String filename)
            throws Exception {

        return loadX509Certificate(filename);
    }

    /**
     * Load an X509Certificate object from a specified file.
     *
     * @param filename The name of the file containing the X.509 Certificate.
     * @return An <code>X509Certificate</code> object.
     * @throws CertificateException If unable to load the Certificate.
     * @throws IOException On IO failure.
     * @throws NoSuchProviderException If the provider is not valid.
     */
    public static X509Certificate loadX509Certificate(String filename)
            throws CertificateException, IOException, NoSuchProviderException {

        InputStream is = ResourceReader.getResourceAsStream(filename);
        CertificateFactory factory = CertificateFactory.getInstance(
                "X.509", "JsafeJCE");
        X509Certificate certificate =
                (X509Certificate) factory.generateCertificate(is);
        is.close();
        return certificate;
    }

    /**
     * Load an X509CRL object from a specified file.
     *
     * @param filename The name of the file containing the X.509 CRL.
     * @return An <code>X509CRL</code> object.
     * @throws CertificateException If unable to load the CRL.
     * @throws IOException On IO failure.
     * @throws NoSuchProviderException If the provider is not valid.
     * @throws CRLException On parsing errors.
     */
    public static X509CRL loadCRl(String filename) throws CertificateException,
            IOException, CRLException, NoSuchProviderException  {

        InputStream is = ResourceReader.getResourceAsStream(filename);
        CertificateFactory factory = CertificateFactory.getInstance(
                "X509", "JsafeJCE");
        X509CRL crl = (X509CRL) factory.generateCRL(is);
        is.close();
        return crl;
    }
}
