/*
 * Copyright (c) 1997-2010 EMC Corporation. All rights reserved.
 *
 * This file shall only be used to demonstrate how to interface to an
 * EMC Corporation licensed development product.
 *
 * You have a royalty-free right to use, reproduce and distribute this
 * demonstration file, provided that you agree that EMC Corporation has no
 * warranty, implied or otherwise, or liability for this demonstration file
 * (including any modified version).  This software is provided "as is"
 * without warranties or representations of any kind. EMC Corporation
 * disclaims all conditions and warranties, statutory and otherwise, both
 * express and implied, with respect to the software, its quality and
 * performance, including but not limited to, all implied warranties of
 * merchantability, fitness for a particular purpose, title and
 * noninfringement of third party rights. Without limiting the foregoing,
 * EMC Corporation does not warrant that the software is error-free or that
 * errors in the product will be corrected. You agree that EMC Corporation
 * shall not be liable for any direct, indirect, incidental, special,
 * consequential, punitive or other damages whatsoever resulting from your use
 * of this software or any modified version.
 *
 */
package jce.common;

import java.math.BigInteger;

import javax.crypto.spec.DHParameterSpec;

import util.Print;

/*
 * Utility class for retrieving Diffie Hellman parameter specification.
 */
public class DHKeyUtil {

    private static final byte[] DH_PRIME = {(byte) 0xF4, (byte) 0x88, (byte) 0xFD, (byte) 0x58, (byte) 0x4E,
            (byte) 0x49, (byte) 0xDB, (byte) 0xCD, (byte) 0x20, (byte) 0xB4, (byte) 0x9D, (byte) 0xE4, (byte) 0x91,
            (byte) 0x07, (byte) 0x36, (byte) 0x6B, (byte) 0x33, (byte) 0x6C, (byte) 0x38, (byte) 0x0D, (byte) 0x45,
            (byte) 0x1D, (byte) 0x0F, (byte) 0x7C, (byte) 0x88, (byte) 0xB3, (byte) 0x1C, (byte) 0x7C, (byte) 0x5B,
            (byte) 0x2D, (byte) 0x8E, (byte) 0xF6, (byte) 0xF3, (byte) 0xC9, (byte) 0x23, (byte) 0xC0, (byte) 0x43,
            (byte) 0xF0, (byte) 0xA5, (byte) 0x5B, (byte) 0x18, (byte) 0x8D, (byte) 0x8E, (byte) 0xBB, (byte) 0x55,
            (byte) 0x8C, (byte) 0xB8, (byte) 0x5D, (byte) 0x38, (byte) 0xD3, (byte) 0x34, (byte) 0xFD, (byte) 0x7C,
            (byte) 0x17, (byte) 0x57, (byte) 0x43, (byte) 0xA3, (byte) 0x1D, (byte) 0x18, (byte) 0x6C, (byte) 0xDE,
            (byte) 0x33, (byte) 0x21, (byte) 0x2C, (byte) 0xB5, (byte) 0x2A, (byte) 0xFF, (byte) 0x3C, (byte) 0xE1,
            (byte) 0xB1, (byte) 0x29, (byte) 0x40, (byte) 0x18, (byte) 0x11, (byte) 0x8D, (byte) 0x7C, (byte) 0x84,
            (byte) 0xA7, (byte) 0x0A, (byte) 0x72, (byte) 0xD6, (byte) 0x86, (byte) 0xC4, (byte) 0x03, (byte) 0x19,
            (byte) 0xC8, (byte) 0x07, (byte) 0x29, (byte) 0x7A, (byte) 0xCA, (byte) 0x95, (byte) 0x0C, (byte) 0xD9,
            (byte) 0x96, (byte) 0x9F, (byte) 0xAB, (byte) 0xD0, (byte) 0x0A, (byte) 0x50, (byte) 0x9B, (byte) 0x02,
            (byte) 0x46, (byte) 0xD3, (byte) 0x08, (byte) 0x3D, (byte) 0x66, (byte) 0xA4, (byte) 0x5D, (byte) 0x41,
            (byte) 0x9F, (byte) 0x9C, (byte) 0x7C, (byte) 0xBD, (byte) 0x89, (byte) 0x4B, (byte) 0x22, (byte) 0x19,
            (byte) 0x26, (byte) 0xBA, (byte) 0xAB, (byte) 0xA2, (byte) 0x5E, (byte) 0xC3, (byte) 0x55, (byte) 0xE9,
            (byte) 0x2F, (byte) 0x78, (byte) 0xC7};

    // X9.42 DH parameters.
    public static final byte[] BASE_G = Print.hexStringToByteArray("8659cf7ac17050012badfa3d2b3af60b469c50a"
            + "61f9b9fde5584eabad825d1550d6389efd1e7b07a71c14fdb89f6cd"
            + "b5274d70113135a6db45d12e49123b8a75cbe909078e9bf89aab9f6"
            + "d15c8c8e9c8e43afb451b3f32217f3571c578783209385ee4aa8fdf"
            + "42eb174ba2d5b5229d8b6067c6b3b5ba685049f8d8159e17f02c");
    public static final byte[] PRIME_P = Print.hexStringToByteArray("c88a97ef46c8eac6089aa153803bc6e58330f87164"
            + "a0864953879130d47c07c0138270314f1cded85133f3a67ac77d3501"
            + "32043c161fbda748de493151e74eb52b43a4a80a6be3c4c946f48e13"
            + "c04a5b40d9f07e3964226aa1e4d26ff5d731007c3816987566d6b33c"
            + "9fc6f65b69298c22feecb26f132b5a647e78885535b3bb");

    public static final byte[] PRIME_Q = Print.hexStringToByteArray("bd5196eddc8a53c818394c2f1e2e80f47e295621");

    public static final BigInteger P = new BigInteger(1, PRIME_P);

    public static final BigInteger G = new BigInteger(1, BASE_G);

    public static final BigInteger Q = new BigInteger(1, PRIME_Q);

    // The SKIP 1024 bit modulus.
    private static final BigInteger SKIP_1024_MODULUS = new BigInteger(1, DH_PRIME);

    // The base used with the SKIP 1024 bit modulus.
    private static final BigInteger SKIP_1024_BASE = BigInteger.valueOf(2);

    /*
     * Get Diffie Hellman parameter specification.
     *
     * @return Diffie Hellman parameter specification. @throws Exception Could not get Diffie Hellman parameter
     * specification.
     */
    public static DHParameterSpec getParamSpec() throws Exception {
        return new DHParameterSpec(SKIP_1024_MODULUS, SKIP_1024_BASE);
    }
}
