/*
 * Copyright (c) 1999-2010 EMC Corporation. All rights reserved.
 *
 * This file shall only be used to demonstrate how to interface to an
 * EMC Corporation licensed development product.
 *
 * You have a royalty-free right to use, reproduce and distribute this
 * demonstration file, provided that you agree that EMC Corporation has no
 * warranty, implied or otherwise, or liability for this demonstration file
 * (including any modified version).  This software is provided "as is"
 * without warranties or representations of any kind. EMC Corporation
 * disclaims all conditions and warranties, statutory and otherwise, both
 * express and implied, with respect to the software, its quality and
 * performance, including but not limited to, all implied warranties of
 * merchantability, fitness for a particular purpose, title and
 * noninfringement of third party rights. Without limiting the foregoing,
 * EMC Corporation does not warrant that the software is error-free or that
 * errors in the product will be corrected. You agree that EMC Corporation
 * shall not be liable for any direct, indirect, incidental, special,
 * consequential, punitive or other damages whatsoever resulting from your use
 * of this software or any modified version.
 *
 */
package util;

/**
 * Print utility class for all the samples.
 */
public final class PrintUtil {

    private PrintUtil() {
        // Prevent instantiation.
    }

    /**
     * Prints out to standard out the current JDK version.
     */
    public static void printJdkVersion() {
        println("JDK version is " + System.getProperty("java.version"));
    }

    /**
     * Print line separator to standard out.
     */
    public static void printLineSeparator() {
        println();
        println("=========================================================");
        println();
    }

    /**
     * Print the specified text to standard output.
     *
     * @param textToPrint The text to be printed.
     */
    public static void print(String textToPrint) {
        System.out.print(textToPrint);
    }

    /**
     * Print a blank line to standard output.
     */
    public static void println() {
        System.out.println();
    }

    /**
     * Print the specified text to standard output.
     *
     * @param textToPrint The text to print.
     */
    public static void println(String textToPrint) {
        System.out.println(textToPrint);
    }

    /**
     * Given a <code>byte</code> array, print out the hex-encoding of that data,
     * in a visually pleasing format.
     *
     * @param byteArray The <code>byte</code> array to be printed.
     */
    public static void printBuffer(byte[] byteArray) {
        printBuffer(byteArray, 0, byteArray.length);
    }

    /**
     * Given a <code>byte</code> array, print the first length bytes of that
     * <code>byte</code> array.
     *
     * @param byteArray The <code>byte</code> array containing the data to be
     *            printed.
     * @param length The amount of data to be printed.
     */
    public static void printBuffer(byte[] byteArray, int length) {
        printBuffer(byteArray, 0, length);
    }

    /**
     * Given some part of a <code>byte</code> array, print the hex-encoding of
     * that data in a visually pleasing format.
     *
     * @param byteArray The <code>byte</code> array containing the data to be
     *            printed.
     * @param offset The starting location of the data to be printed.
     * @param length The amount of data to be printed.
     */
    public static void printBuffer(byte[] byteArray, int offset, int length) {

        StringBuffer textLine = new StringBuffer("                ");
        print("  0000: ");
        for (int i = 0; i < length; i++) {
            if (i % 16 == 0 && i != 0) {
                println("[" + textLine + "]");
                print("  " + hexString(i, 4) + ": ");
                for (int j = 0; j < 16; j++) {
                    textLine.setCharAt(j, ' ');
                }
            }
            print(hexString(byteArray[i + offset], 2) + " ");
            if (byteArray[i + offset] < 32 || byteArray[i + offset] > 127 ||
                    byteArray[i + offset] == 0x7f) {
                textLine.setCharAt(i % 16, '.');
            } else {
                textLine.setCharAt(i % 16, (char) byteArray[i + offset]);
            }
        }
        if (length % 16 != 0 || length == 0) {
            for (int i = 0; i < 16 - length % 16; i++) {
                print("   ");
            }
        }
        println("[" + textLine + "]");
    }

    private static String hexString(int value, int padding) {

        String hexString = "0123456789ABCDEF";
        StringBuffer tempString = new StringBuffer("                                                                              ".substring(0, padding));
        int offset = padding - 1;

        for (int i = 0; i < padding; i++) {
            tempString.setCharAt(offset - i,
                    hexString.charAt(value >> i * 4 & 0xF));
        }
        return tempString.toString();
    }

    public static void printSampleStart(String sample) {
        println("Beginning " + sample + " sample.");
    }
}
