/* GCMMode.java
 * Copyright (c) 1997-2009, RSA Security Inc.
 *
 * This file is used to demonstrate how to interface to an RSA
 * Security licensed development product.  You have a
 * royalty-free right to use, modify, reproduce and distribute this
 * demonstration file (including any modified version), provided that
 * you agree that RSA Security has no warranty, implied or
 * otherwise, or liability for this demonstration file or any modified
 * version.
 *
 */

package crypto;

import javax.crypto.Cipher;
import javax.crypto.SecretKey;
import javax.crypto.spec.IvParameterSpec;

import jce.common.AESKeyUtil;
import jce.common.JCESample;
import util.Array;
import util.Print;
import util.SampleFailedException;

import com.rsa.jsafe.provider.SensitiveData;

/*
 * This application demonstrates the use of RSA Security's JCE provider to
 * utilize AES (Advanced Encryption Standard) using GCM (Galois/Counter mode).
 */
public class GCMMode extends JCESample {

    /*
     * This is the initialization vector data used for encryption
     * and decryption.
     *
     * In order to set the three parameters authentication tag length,
     * authenticated data length and GCM algorithm IV, use the
     * following special 21-byte format for the IV:
     *
     *    1. The first byte is the authentication tag length
     *       (between 12 and 16 inclusive).
     *    2. The next eight bytes represent the length of the
     *       authenticated data (big endian).
     *    3. The remaining 12 bytes specify the GCM algorithm IV.
     *
     *    NOTE: It is critically important that the 12-byte GCM
     *          algorithm IV used for a given key choice is
     *          unique. To achieve this, one could generate the
     *          required 12-bytes using an approved SecureRandom.
     *
     * For example, if the authentication tag length length is 12,
     * the length of the authenticated data is 10, and the 12-byte
     * unique GCM algorithm IV is [10 11 12 13 14 15 16 17 18 19 20 21],
     * then the byte array to pass into the AlgorithmParameters.init()
     * method would be:
     *   [08 00 00 00 00 00 00 00 0a 10 11 12 13 14 15 16 17 18 19 20 21]
     */
    private static final byte[] INITIALIZATION_VECTOR = {

            // This is the authentication tag length (12).
            (byte) 0x0c,

            // This is the length of the authenticated data (10).
            (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00,
            (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x0a,

            // This is the GCM IV data used for encryption and decryption.
            // These 12-bytes MUST be unique for a key choice
            // for security reasons.
            (byte) 0x10, (byte) 0x11, (byte) 0x12, (byte) 0x13,
            (byte) 0x14, (byte) 0x15, (byte) 0x16, (byte) 0x17,
            (byte) 0x18, (byte) 0x19, (byte) 0x20, (byte) 0x21,
    };

    public static void main(String[] args) throws Exception {
        GCMMode aesEncrypter = new GCMMode();
        aesEncrypter.runSample();
    }

    public void runSample() throws Exception {

        Print.jdkVersion();

        // The encryption object.
        Cipher aesEncrypter = null;

        // The decryption object.
        Cipher aesDecrypter = null;

        // The counter mode parameters.
        IvParameterSpec ivParamSpec;

        // The secret key object.
        SecretKey secretKey;

        // A byte array to hold the encrypted data.
        byte[] ciphertext;

        // A byte array to hold the decrypted data.
        byte[] recoveredPlaintext;

        Print.beginSample("JCE GCMMode Encryption and Decryption");

        // This is the plaintext data to encrypt.
        byte[] plaintext = new String("This is my plaintext ååååå There are many variations of passages of Lorem Ipsum available, but the majority have suffered alteration in some form, by injected humour, or randomised words which don't look even slightly believable. If you are going to use a passage of Lorem Ipsum, you need to be sure there isn't anything embarrassing hidden in the middle of text. All the Lorem Ipsum generators on the Internet tend to repeat predefined chunks as necessary, making this the first true generator on the Internet. It uses a dictionary of over 200 Latin words, combined with a handful of model sentence structures, to generate Lorem Ipsum which looks reasonable. The generated Lorem Ipsum is therefore always free from repetition, injected humour, or non-characteristic words etc.").getBytes();
        Print.plaintext(plaintext);

        // Add the JsafeJCE provider.
        Print.println("Creating and adding JsafeJCE provider.");
        addJsafeJCE();

        // Print out a list of the current providers.
        printRegisteredProviders();

        // Get the default AES key.
        secretKey = AESKeyUtil.getSecretKey();

        Print.keyData("AES key data (128 bits)", secretKey.getEncoded());

        // Set the AES IV.
        ivParamSpec = new IvParameterSpec(INITIALIZATION_VECTOR);
        Print.iv(INITIALIZATION_VECTOR);

        try {
            // Get a cipher object for AES encryption with Galois Counter Mode
            // and no padding.  Note that GCM mode is like a stream cipher, so
            // no padding is needed, even if the plaintext length is not a
            // multiple of the block size.
            aesEncrypter = Cipher.getInstance("AES/GCM/NoPadding", "JsafeJCE");

            // Initialize the encryption object.  This call will
            // set up aesEncrypter to perform encryption using AES.
            aesEncrypter.init(Cipher.ENCRYPT_MODE, secretKey, ivParamSpec);

            // Create some space to hold the results of
            // encrypting the plaintext.  A call to getOutputSize() will
            // return the number of bytes necessary to hold the encrypted
            // data.  This number may be larger than the number that is
            // actually needed.
            ciphertext =
                    new byte[aesEncrypter.getOutputSize(plaintext.length)];

            // First, input the additional authenticated data. The method
            // update() first expects authDataLength bytes of input data.
            // This is processed as the additional authenticated data. The
            // number of bytes expected should have been specified in the
            // ivParamSpec input to init(). There will be no output bytes
            // for input of authenticated data.
            Print.println("Processing authenticated data.");

            // This is the optional additional data that is authenticated
            // but not encrypted.
            byte[] authenticatedData = new byte[10];
            int outputLenUpdate = aesEncrypter.update(authenticatedData,
                    0, authenticatedData.length, ciphertext, 0);

            // Now, encrypt the data.  The call to update() will
            // encrypt as much data as is possible. Since GCM mode
            // is like a stream cipher, even if the length of
            // plaintext does not contain enough bytes to fill
            // a whole block, the extra bytes will still be encrypted.
            outputLenUpdate += aesEncrypter.update(plaintext, 0,
                    plaintext.length, ciphertext, outputLenUpdate);

            // Call doFinal() after passing in all of the data to encrypt.
            // In GCM, doFinal() will output an authentication tag based
            // on the all the input data, apply any necessary padding to the
            // input data, and finish the encryption.
            int outputLenFinal =
                    aesEncrypter.doFinal(ciphertext, outputLenUpdate);

            int outputLenTotal = outputLenUpdate + outputLenFinal;

            // Print out the encrypted data.
            Print.ciphertext(ciphertext, outputLenTotal);

            // Decrypt data.

            // Get a cipher object for AES decryption with Galois/Counter Mode
            // and no padding.
            aesDecrypter = Cipher.getInstance("AES/GCM/NoPadding", "JsafeJCE");

            // Initialize the decryption object.  This call will
            // set up aesDecrypter to perform decryption using AES.
            // Note: the ivParamSpec still contains the same parameters
            //    as used in the encrypt phase (this allows decrypt to match).
            aesDecrypter.init(Cipher.DECRYPT_MODE, secretKey, ivParamSpec);

            // It's necessary to create some space to hold the results
            // of decrypting the ciphertext.  A call to getOutputSize()
            // will return the number of bytes necessary to hold the
            // decrypted data.  This number may be larger than the number
            // that is actually needed.
            recoveredPlaintext =
                    new byte[aesDecrypter.getOutputSize(ciphertext.length)];

            // First, input the additional authenticated data. The method
            // update() first expects authDataLength bytes of input data.
            // This is processed as the additional authenticated data.
            // The number of bytes expected should have been specified in the
            // ivParamSpec input to init().
            Print.println("Processing authenticated data.");
            outputLenUpdate = aesDecrypter.update(authenticatedData,
                    0, authenticatedData.length, recoveredPlaintext, 0);

            // Decrypt the data.  The call to update() will decrypt as
            // much data as is possible.  If not enough bytes are passed in
            // to decrypt a whole block, the extra bytes will be kept for
            // the next call to update(), or a call to doFinal().
            outputLenUpdate += aesDecrypter.update(ciphertext, 0,
                    outputLenTotal, recoveredPlaintext, outputLenUpdate);

            // After passing in all of the data to decrypt,
            // call doFinal().  In GCM, doFinal() will verify the
            // authenticity of the input data, and will throw an
            // exception if the verification failed.
            // Otherwise, it will finish the decryption.
            outputLenFinal =
                    aesDecrypter.doFinal(recoveredPlaintext, outputLenUpdate);
            outputLenTotal = outputLenUpdate + outputLenFinal;

            // Print out the recovered plaintext.
            Print.recoveredPlaintext(recoveredPlaintext, outputLenTotal);
            System.out.println("Min melding er: " + new String(recoveredPlaintext).substring(0, recoveredPlaintext.length - 29));

            // Check to make sure that the input plaintext is the same
            // as the output recovered plaintext.
            if (!Array.equals(plaintext, plaintext.length,
                    recoveredPlaintext, outputLenTotal)) {
                throw new SampleFailedException(
                        Print.RECOVERED_PLAINTEXT_DOES_NOT_MATCH_ORIGINAL);
            }

            Print.successfulEnding();

        } finally {

            // Cryptographic objects should be cleared once they are no longer
            // needed.
            SensitiveData.clear(aesEncrypter);
            SensitiveData.clear(aesDecrypter);
            SensitiveData.clear(secretKey);
        }
    }
}