/*
 * Copyright (c) 1997-2010 EMC Corporation. All rights reserved.
 *
 * This file shall only be used to demonstrate how to interface to an
 * EMC Corporation licensed development product.
 *
 * You have a royalty-free right to use, reproduce and distribute this
 * demonstration file, provided that you agree that EMC Corporation has no
 * warranty, implied or otherwise, or liability for this demonstration file
 * (including any modified version).  This software is provided "as is"
 * without warranties or representations of any kind. EMC Corporation
 * disclaims all conditions and warranties, statutory and otherwise, both
 * express and implied, with respect to the software, its quality and
 * performance, including but not limited to, all implied warranties of
 * merchantability, fitness for a particular purpose, title and
 * noninfringement of third party rights. Without limiting the foregoing,
 * EMC Corporation does not warrant that the software is error-free or that
 * errors in the product will be corrected. You agree that EMC Corporation
 * shall not be liable for any direct, indirect, incidental, special,
 * consequential, punitive or other damages whatsoever resulting from your use
 * of this software or any modified version.
 *
 */
package jce.common;

import java.math.BigInteger;
import java.security.KeyFactory;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.spec.DSAParameterSpec;
import java.security.spec.PKCS8EncodedKeySpec;
import java.security.spec.X509EncodedKeySpec;

import util.Print;

/*
 * Utility class for retrieving DSA parameter specification and DSA key pair.
 */
public class DSAKeyUtil {

    private static final byte[] PRIME_P = Print
            .hexStringToByteArray("db2bdd5cd5a185612425a47ef3074c02da84760179420" +
                    "c8ae869ee7dc8b2ea7689c1d3055695f4a429713a5b6dfd1ba8fbc5f24a" +
                    "9c0632bf3838e6e2e719c684f172fd18fd9c14711bb8cf92377ae85b18" +
                    "125ef62229b8c6fc7155fe90aba53835063be8989b5e84212eac163743" +
                    "b4581f02cb77791230016792932a72c3d3bb");

    private static final byte[] SUB_PRIME_Q = Print.hexStringToByteArray(
            "99e50af740b7a4a1449b4de976da7283ac663763");

    private static final byte[] BASE_G = Print
            .hexStringToByteArray("2f79170803cb1c971c4f72799441782446eb3e8bd1205" +
                    "dbb5118a39c9225e9fd5bcdefcdcc3eacba917e6ed47f7ce1285c2079ac" +
                    "b20e2c158ba9e52ac2299b6a56c3b8903fec5b731adbe8b4e0938d32e0c" +
                    "74568a0162891fba276d7d066a0f9e2adb919a05e35caf08cc1af76129c" +
                    "847fc2ab75e41ce253986091a1e957beba");

    private static final byte[] DSA_PUBLIC_KEY_BER = Print
            .hexStringToByteArray("308201b73082012b06072a8648ce3804013082011e028" +
                    "18100db2bdd5cd5a185612425a47ef3074c02da84760179420c8ae869ee" +
                    "7dc8b2ea7689c1d3055695f4a429713a5b6dfd1ba8fbc5f24a9c0632bf3" +
                    "838e6e2e719c684f172fd18fd9c14711bb8cf92377ae85b18125ef6222" +
                    "9b8c6fc7155fe90aba53835063be8989b5e84212eac163743b4581f02cb" +
                    "77791230016792932a72c3d3bb02150099e50af740b7a4a1449b4de976d" +
                    "a7283ac6637630281802f79170803cb1c971c4f72799441782446eb3e8b" +
                    "d1205dbb5118a39c9225e9fd5bcdefcdcc3eacba917e6ed47f7ce1285c2" +
                    "079acb20e2c158ba9e52ac2299b6a56c3b8903fec5b731adbe8b4e0938d" +
                    "32e0c74568a0162891fba276d7d066a0f9e2adb919a05e35caf08cc1af7" +
                    "6129c847fc2ab75e41ce253986091a1e957beba0381850002818100cdbb" +
                    "b72c8516f9660630a32368e6867b69c59900281ea6dfacd700ff719bf89" +
                    "60ff0878d8c72da334dd90d8a1e897a3cdd8aa1321a91f82d842a391c78" +
                    "b8175f2784217e7ccf987df7b1d4cff54472feb9d1885dd5581fe6d81ec" +
                    "ed0039eb3c5578f0aa1247131e48df6b329c705612ccf23192db45f65e7" +
                    "de119b494339d9f4");

    private static final byte[] DSA_PRIVATE_KEY_BER = Print
            .hexStringToByteArray("3082014a0201003082012b06072a8648ce38040130820" +
                    "11e02818100db2bdd5cd5a185612425a47ef3074c02da84760179420c8a" +
                    "e869ee7dc8b2ea7689c1d3055695f4a429713a5b6dfd1ba8fbc5f24a9c0" +
                    "632bf3838e6e2e719c684f172fd18fd9c14711bb8cf92377ae85b18125e" +
                    "f62229b8c6fc7155fe90aba53835063be8989b5e84212eac163743b4581" +
                    "f02cb77791230016792932a72c3d3bb02150099e50af740b7a4a1449b4d" +
                    "e976da7283ac6637630281802f79170803cb1c971c4f72799441782446e" +
                    "b3e8bd1205dbb5118a39c9225e9fd5bcdefcdcc3eacba917e6ed47f7ce1" +
                    "285c2079acb20e2c158ba9e52ac2299b6a56c3b8903fec5b731adbe8b4e" +
                    "0938d32e0c74568a0162891fba276d7d066a0f9e2adb919a05e35caf08c" +
                    "c1af76129c847fc2ab75e41ce253986091a1e957beba041602142731891" +
                    "d8cd4b5df7c363e4f9aa441b9d315a504");

    /*
     * Get a pre-generated DSA parameter specification.
     *
     * @return The pre-generated DSA parameter specification.
     * @throws Exception Could not get DSA parameter specification.
     */
    public static DSAParameterSpec getDSAParameterSpec() throws Exception {
        return new DSAParameterSpec(new BigInteger(PRIME_P),
                new BigInteger(SUB_PRIME_Q), new BigInteger(BASE_G));
    }

    /*
     * Get the public key of the pre-generated DSA key pair.
     *
     * @return The pre-generated public key.
     * @throws Exception Could not get the public key.
     */
    public static PublicKey getPublic() throws Exception {

        X509EncodedKeySpec pubKeySpec =
                new X509EncodedKeySpec(DSA_PUBLIC_KEY_BER);
        KeyFactory factory = KeyFactory.getInstance("DSA", "JsafeJCE");
        return factory.generatePublic(pubKeySpec);
    }

    /*
     * Get the private key of the pre-generated DSA key pair.
     *
     * @return The pre-generated private key.
     * @throws Exception Could not get the private key.
     */
    public static PrivateKey getPrivate() throws Exception {

        PKCS8EncodedKeySpec privKeySpec =
                new PKCS8EncodedKeySpec(DSA_PRIVATE_KEY_BER);
        KeyFactory factory = KeyFactory.getInstance("DSA", "JsafeJCE");
        return factory.generatePrivate(privKeySpec);
    }
}
