/*
 * Copyright (c) 1997-2010 EMC Corporation. All rights reserved.
 *
 * This file shall only be used to demonstrate how to interface to an
 * EMC Corporation licensed development product.
 *
 * You have a royalty-free right to use, reproduce and distribute this
 * demonstration file, provided that you agree that EMC Corporation has no
 * warranty, implied or otherwise, or liability for this demonstration file
 * (including any modified version).  This software is provided "as is"
 * without warranties or representations of any kind. EMC Corporation
 * disclaims all conditions and warranties, statutory and otherwise, both
 * express and implied, with respect to the software, its quality and
 * performance, including but not limited to, all implied warranties of
 * merchantability, fitness for a particular purpose, title and
 * noninfringement of third party rights. Without limiting the foregoing,
 * EMC Corporation does not warrant that the software is error-free or that
 * errors in the product will be corrected. You agree that EMC Corporation
 * shall not be liable for any direct, indirect, incidental, special,
 * consequential, punitive or other damages whatsoever resulting from your use
 * of this software or any modified version.
 *
 */
package jce.common;

import java.math.BigInteger;
import java.security.KeyFactory;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.spec.PKCS8EncodedKeySpec;
import java.security.spec.RSAPrivateCrtKeySpec;
import java.security.spec.RSAPrivateKeySpec;
import java.security.spec.RSAPublicKeySpec;

import util.Print;
import util.ResourceReader;

/*
 * Utility class for retrieving pre-generated RSA key pair.
 */
public class RSAKeyUtil {

    // The following data is for a pre-generated 1024 bit RSA key pair.
    private static final byte[] N_DATA = Print.hexStringToByteArray(
        "00ee974391eb68b3ab5a67003ccdbcb6f78afe52402608f5c502a5e53d6b452e92" +
        "dc965bb8fcaccec3ba4876065fffa7794e9c01d6589a911f6123877b84fa83b838" +
        "368415f296ec8807ddf81cb99037d1263dc512b42a643ce0ef051233928d7c787c" +
        "4babe6d445c90a842c79b808ccb38dc256129579b4fea395c2f7608836b7");

    private static final byte[] E_DATA = Print.hexStringToByteArray(
        "010001");

    private static final byte[] D_DATA = Print.hexStringToByteArray(
        "27e3410829a790f612408b111f4230029589ac1d8e5e4c1c5a337aee383b522c7a" +
        "5566d337e6fa104dd9e4656f7231c0fd8be660bbe26765ac1b4100bcfc16be087d" +
        "b41115a5bfbd5619d78315b203560a6a7a5ae1be089dc83a64dd83927cc6e40a22" +
        "652749eafffa5879ea7a0f98f28cf99d3a0a986e834746b7e3b8b3df39");

    private static final byte[] P_DATA = Print.hexStringToByteArray(
        "00fb8f2c35415d06a11aa97c45c4dec22dc8fc6f69e8f8fb38965c817fe543831e" +
        "11df267da392be9d5292beb9f26223515bd669281f7088de6ff0f0cc804e3bbf");

    private static final byte[] Q_DATA = Print.hexStringToByteArray(
        "00f2cd7c49182747582561de2c6bad859268308d577b167040a8a2de0b1de26e65" +
        "cfd68e0c61057f0ac9fb6645fb5d9f40573b59b93f20dfae8f0660d01fa32309");

    private static final byte[] EXP_P_DATA = Print.hexStringToByteArray(
        "00ee61c66ad0fdee824c59bfc1fdc57a0d1920de13670a4a465d765be16ec8833f" +
        "c37fcb61d3d80379e1fddd6f2819878e5990dd2580658bcdfe163c9a0e1c14d1");

    private static final byte[] EXP_Q_DATA = Print.hexStringToByteArray(
        "407a8952eff31e632e2a3b521896502e7d34c7943fbfc9b7a210fe5e7d65306421" +
        "399d27746c7d3c643f3afdc2af2e276c02fae4d2faebebf357824642e093e1");

    private static final byte[] CRT_COEF_DATA = Print.hexStringToByteArray(
        "41fa97502f235fe8d14abd2afb68ca3c24e5364653f0af9cab4113a93403a86965" +
        "7e3a3c6b93760d487bf07e75caaab78387830a8992067ead68636c63c8954c");

    /*
     * Get the public key of the pre-generated RSA key pair.
     *
     * @return The pre-generated public key.
     * @throws Exception Could not get the public key.
     */
    public static PublicKey getPublic() throws Exception {

        RSAPublicKeySpec publicKeySpec = new RSAPublicKeySpec(
                new BigInteger(1, N_DATA), new BigInteger(1, E_DATA));

        KeyFactory kf = KeyFactory.getInstance("RSA", "JsafeJCE");
        return kf.generatePublic(publicKeySpec);
    }

    /*
     * Get the private key of the pre-generated RSA key pair.
     *
     * @return The pre-generated private key.
     * @throws Exception Could not get the private key.
     */
    public static PrivateKey getPrivate() throws Exception {

        RSAPrivateKeySpec privateKeySpec = new RSAPrivateCrtKeySpec(
            new BigInteger(1, N_DATA), new BigInteger(1, E_DATA),
            new BigInteger(1, D_DATA), new BigInteger(1, P_DATA),
            new BigInteger(1, Q_DATA), new BigInteger(1, EXP_P_DATA),
            new BigInteger(1, EXP_Q_DATA), new BigInteger(1, CRT_COEF_DATA));

        KeyFactory kf = KeyFactory.getInstance("RSA", "JsafeJCE");
        return kf.generatePrivate(privateKeySpec);
    }

    public static PrivateKey loadRawPrivate(String file) throws Exception {

        // Create the PrivateKey using the JsafeJCE provider.
        KeyFactory kf = KeyFactory.getInstance("RSA", "JsafeJCE");
        PKCS8EncodedKeySpec encodedKeySpec = new PKCS8EncodedKeySpec(
                ResourceReader.readResource(file));
        return kf.generatePrivate(encodedKeySpec);
    }
}
